/*
 *  This file is part of MONGKIE. Visit <http://www.mongkie.org/> for details.
 *  Copyright (C) 2012 Korean Bioinformation Center(KOBIC)
 * 
 *  MONGKIE is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  MONGKE is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 * 
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.mongkie.ui.datatable.graph;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.AbstractAction;
import javax.swing.JComponent;
import org.jdesktop.swingx.JXSearchField;
import org.mongkie.datatable.spi.DataTable;
import org.mongkie.filter.FilterController;
import org.mongkie.filter.spi.Filter;
import org.mongkie.visualization.search.SearchController;
import org.mongkie.visualization.search.SearchOption;
import org.openide.util.Lookup;
import prefuse.data.Schema;
import prefuse.data.Tuple;

/**
 *
 * @author Yeongjun Jang <yjjang@kribb.re.kr>
 */
class FilterToolsPanel extends javax.swing.JPanel implements DataTable.Tool<AbstractDataTable>, ItemListener {

    private AbstractDataTable table;
    private static final String NONE = "---None";
    private static final String ALL_COLUMNS = "---All columns";
    private RegexFilter filter;

    /**
     * Creates new form FilterToolsPanel
     */
    FilterToolsPanel(AbstractDataTable table) {
        this.table = table;
        initComponents();

        ((JXSearchField) filterInputTextField).setCancelAction(
                new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        clearFilter(false);
                    }
                });
        ((JXSearchField) filterInputTextField).setAction(
                new AbstractAction() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        if (filter != null) {
                            applyFilter(false);
                        }
                    }
                });
        filterInputTextField.addKeyListener(new KeyAdapter() {
            @Override
            public void keyReleased(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    applyFilter(true); // Re-apply on ENTER KEY
                }
            }
        });
    }

    private void clearFilter(boolean reapply) {
        filterInputTextField.setText(null);
        if (!filter.setText(null) && reapply) {
            filter.reapply();
        }
    }

    private void applyFilter(boolean reapply) {
        String text = filterInputTextField.getText();
        if (!filter.setText(text.isEmpty() ? null : text) && reapply) {
            filter.reapply();
        }
    }

    /**
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        filterLabel = new javax.swing.JLabel();
        filterInputTextField = new JXSearchField();
        ((JXSearchField) filterInputTextField).setLayoutStyle(JXSearchField.LayoutStyle.VISTA);
        ((JXSearchField) filterInputTextField).setUseNativeSearchFieldIfPossible(false);
        ((JXSearchField) filterInputTextField).setPromptBackround(java.awt.Color.white);
        filterColumnComboBox = new javax.swing.JComboBox();

        setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        setFocusable(false);
        setRequestFocusEnabled(false);

        filterLabel.setFont(new java.awt.Font("Tahoma", 0, 12)); // NOI18N
        filterLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/mongkie/ui/datatable/resources/funnel.png"))); // NOI18N
        filterLabel.setText(org.openide.util.NbBundle.getMessage(FilterToolsPanel.class, "FilterToolsPanel.filterLabel.text")); // NOI18N

        filterInputTextField.setFont(new java.awt.Font("Tahoma", 0, 12)); // NOI18N
        filterInputTextField.setText(org.openide.util.NbBundle.getMessage(FilterToolsPanel.class, "FilterToolsPanel.filterInputTextField.text")); // NOI18N
        filterInputTextField.setEnabled(false);
        filterInputTextField.setPreferredSize(new java.awt.Dimension(100, 24));

        filterColumnComboBox.setFont(new java.awt.Font("Tahoma", 0, 12)); // NOI18N
        filterColumnComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "NONE" }));
        filterColumnComboBox.setEnabled(false);
        filterColumnComboBox.setPreferredSize(new java.awt.Dimension(100, 24));

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(filterLabel)
                .addGap(8, 8, 8)
                .addComponent(filterInputTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 145, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(filterColumnComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 115, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                .addComponent(filterColumnComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(filterInputTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(filterLabel))
        );

        layout.linkSize(javax.swing.SwingConstants.VERTICAL, new java.awt.Component[] {filterColumnComboBox, filterInputTextField});

    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox filterColumnComboBox;
    private javax.swing.JTextField filterInputTextField;
    private javax.swing.JLabel filterLabel;
    // End of variables declaration//GEN-END:variables

    @Override
    public AbstractDataTable getDataTable() {
        return table;
    }

    @Override
    public JComponent getComponent() {
        return this;
    }

    private String getFilterName() {
        return RegexFilter.class.getName() + "_" + table.getDataGroup();
    }

    @Override
    public void refresh(boolean disabled) {
        if (disabled
                || table.getModel().getDisplay().getDataViewSupport(table.getDataGroup()).getOutlineSchema().getColumnCount() == 0) {
            filterColumnComboBox.removeItemListener(this);
            filterColumnComboBox.removeAllItems();
            filterColumnComboBox.addItem(NONE);
            filterColumnComboBox.setEnabled(false);
            filterInputTextField.setText(null);
            filterInputTextField.setEnabled(false);
            filter = null;
        } else {
            AbstractDataTable.AbstractModel model = table.getModel();
            Schema s = model.getDisplay().getDataViewSupport(table.getDataGroup()).getOutlineSchema();
            filterColumnComboBox.removeItemListener(this);
            filterColumnComboBox.removeAllItems();
            filterColumnComboBox.addItem(ALL_COLUMNS);
            for (int i = 0; i < s.getColumnCount(); i++) {
                String col = s.getColumnName(i);
                if (model.getTable().canGetString(col)) {
                    filterColumnComboBox.addItem(col);
                }
            }
            filter = (RegexFilter) Lookup.getDefault().lookup(FilterController.class).getModel(model.getDisplay()).getFilter(table.getDataGroup(), getFilterName());
            if (filter == null) {
                Lookup.getDefault().lookup(FilterController.class).addFilter(filter = new RegexFilter(table, getFilterName()), true);
            }
            filterColumnComboBox.setSelectedItem(filter.getColumn());
            Schema old = (Schema) model.getTable().getClientProperty(Schema.class.getName());
            if (s != old) { // Some columns are added or deleted
                model.getTable().putClientProperty(Schema.class.getName(), s);
                String col = (String) filterColumnComboBox.getSelectedItem();
                if (!filter.getColumn().equals(col) // Filter column was deleted
                        || col.equals(ALL_COLUMNS)) {
                    // setColumn() results in applying the filter
                    filter.column = col;
                    clearFilter(true); // Apply the filter only once
                } else {
                    filterInputTextField.setText(filter.getText());
                    applyFilter(true);
                }
            } else {
                filterInputTextField.setText(filter.getText());
            }
            filterColumnComboBox.addItemListener(this);
            filterColumnComboBox.setEnabled(true);
            filterInputTextField.setEnabled(true);
        }
    }

    @Override
    public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
            filter.setColumn((String) e.getItem());
        }
    }

    private static class RegexFilter extends Filter {

        private String column = ALL_COLUMNS, text = null;
        private final String name;
        private AbstractDataTable table;

        RegexFilter(AbstractDataTable table, String name) {
            this.table = table;
            this.name = name;
        }

        String getColumn() {
            return column;
        }

        boolean setColumn(String column) {
            if (column.equals(this.column)) {
                return false;
            }
            this.column = column;
            if (text != null) {
                fireExpressionChange(); // Apply the filter
            }
            return true;
        }

        String getText() {
            return text;
        }

        boolean setText(String text) {
            if ((text != null && text.equals(this.text))
                    || (text == null && this.text == null)) {
                return false;
            }
            this.text = text;
            fireExpressionChange(); // Apply the filter
            return true;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public String getDataGroup() {
            return table.getDataGroup();
        }

        @Override
        public boolean getBoolean(Tuple data) {
            if (text == null) {
                return true;
            }
            SearchController searcher = Lookup.getDefault().lookup(SearchController.class);
            return searcher.match(data, searcher.makeRegexPattern(text, SearchOption.getDefault()), getFilterColumns());
        }

        private String[] getFilterColumns() {
            if (column.equals(ALL_COLUMNS)) {
                Schema s = table.getModel().getDisplay().getDataViewSupport(table.getDataGroup()).getOutlineSchema();
                String[] columns = new String[s.getColumnCount()];
                for (int i = 0; i < s.getColumnCount(); i++) {
                    String col = s.getColumnName(i);
                    if (table.getModel().getTable().canGetString(col)) {
                        columns[i] = col;
                    }
                }
                return columns;
            }
            return new String[]{column};
        }
    }
}
