/*
 * This file is part of MONGKIE. Visit <http://www.mongkie.org/> for details.
 * Copyright (C) 2011 Korean Bioinformation Center(KOBIC)
 * 
 * MONGKIE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * MONGKIE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * SearchPanel.java
 *
 * Created on Jul 10, 2011, 3:55:56 AM
 */
package org.mongkie.ui.visualization.options;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.AbstractAction;
import kobic.prefuse.display.DisplayListener;
import kobic.prefuse.display.NetworkDisplay;
import org.mongkie.visualization.MongkieDisplay;
import prefuse.data.Graph;

/**
 *
 * @author Yeongjun Jang <yjjang@kribb.re.kr>
 */
public class InstantSearchPanel extends javax.swing.JPanel {

    private final MongkieDisplay display;

    /**
     * Creates new form SearchPanel
     */
    public InstantSearchPanel(MongkieDisplay d) {
        this.display = d;
        initComponents();

        jXSearchField.setCancelAction(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                jXSearchField.setText(null);
                display.searchNodes(null);
            }
        });
//        jXSearchField.setFindAction(new ActionListener() {
//
//            @Override
//            public void actionPerformed(ActionEvent e) {
//                display.searchNodes(e.getActionCommand());
//            }
//        });
        jXSearchField.setAction(new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent e) {
                display.searchNodes(e.getActionCommand());
            }
        });
        jXSearchField.setEnabled(d.getGraph().getNodeCount() > 0);
        d.addDisplayListener(new DisplayListener() {
            @Override
            public void graphDisposing(NetworkDisplay d, Graph g) {
                jXSearchField.setEnabled(false);
            }

            @Override
            public void graphChanged(NetworkDisplay d, Graph g) {
                jXSearchField.setEnabled(g.getNodeCount() > 0);
            }
        });
    }

    ActionListener getCancelAction() {
        return jXSearchField.getCancelAction();
    }

    void setTextField(String text) {
        jXSearchField.setText(text);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jXSearchField = new org.jdesktop.swingx.JXSearchField();

        jXSearchField.setLayoutStyle(org.jdesktop.swingx.JXSearchField.LayoutStyle.VISTA);
        jXSearchField.setText(org.openide.util.NbBundle.getMessage(InstantSearchPanel.class, "InstantSearchPanel.jXSearchField.text", new Object[] {})); // NOI18N
        jXSearchField.setUseNativeSearchFieldIfPossible(false);
        jXSearchField.setFont(new java.awt.Font("Tahoma", 0, 12)); // NOI18N
        jXSearchField.setPreferredSize(new java.awt.Dimension(240, 24));
        jXSearchField.setPromptBackround(java.awt.Color.white);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jXSearchField, javax.swing.GroupLayout.PREFERRED_SIZE, 240, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addGap(0, 0, Short.MAX_VALUE)
                .addComponent(jXSearchField, javax.swing.GroupLayout.PREFERRED_SIZE, 24, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private org.jdesktop.swingx.JXSearchField jXSearchField;
    // End of variables declaration//GEN-END:variables
}
